/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.sinfo.flood_duration;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.common.GeneralResultType;
import org.dive4elements.river.artifacts.common.ResultRow;
import org.dive4elements.river.artifacts.model.BlackboardDataFacet;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.sinfo.common.SInfoResultType;
import org.dive4elements.river.model.sinfo.Infrastructure;
import org.dive4elements.river.model.sinfo.InfrastructureValue;

/**
 * Facet for a infrastructure value series loaded from the database
 *
 * @author Matthias Schäfer
 */
public class InfrastructureFacet extends BlackboardDataFacet implements FacetTypes {

    private static final long serialVersionUID = 1;

    public InfrastructureFacet(final String name, final String description, final String yAxisLabel) {
        super(0, name, description);

        this.metaData.put("X", "chart.longitudinal.section.xaxis.label");
        this.metaData.put("Y", yAxisLabel);
    }

    /**
     * Returns the data this facet requires.
     *
     * @param artifact
     *            the owner artifact.
     * @param context
     *            the CallContext (ignored).
     *
     * @return
     *         the data as InfrastructureQueryCalculationResult
     */
    @Override
    public Object getData(final Artifact artifact, final CallContext context) {

        final InfrastructureAccess access = new InfrastructureAccess((D4EArtifact) artifact);
        final Infrastructure series = Infrastructure.getSeries(access.getId());
        final List<InfrastructureValue> values = InfrastructureValue.getValues(series, access.getFrom(true), access.getTo(true));
        final Collection<ResultRow> rows = new ArrayList<>();
        for (final InfrastructureValue value : values) {
            rows.add(ResultRow.create().putValue(GeneralResultType.station, value.getStation()) //
                    .putValue(SInfoResultType.infrastructureHeight, value.getHeight()));
        }
        return new InfrastructureQueryCalculationResult(series.getFilename(), rows);
    }

    /**
     * Create a deep copy of this Facet.
     *
     * @return a deep copy.
     */
    @Override
    public InfrastructureFacet deepCopy() {
        final InfrastructureFacet copy = new InfrastructureFacet(this.name, this.description, this.metaData.get("Y"));
        copy.set(this);
        return copy;
    }
}